#!/usr/bin/perl -w
##################################################
# Syntax: includer [-i includedir] directory
##################################################

use Getopt::Std;
use File::Find;
use Cwd;
use strict;
                         
my (%INCLUDE_MAP, $INCLUDE_ROOT);        # Globals

my %opt;
getopts('ri:', \%opt) || usage("Argument Error");

print "READONLY MODE\n" if $opt{r};

my $include_dir = $opt{i} || "include";

my $now = cwd();                # Get absolute path
chdir($include_dir) || 
    usage("Cannot include from $include_dir");
$INCLUDE_ROOT = cwd();
chdir($now);

usage("No start directory given") if $#ARGV < 0;

usage("Start directory doesn't exist: $ARGV[0]")
    unless -d $ARGV[0];

File::Find::find(\&scan_include, $INCLUDE_ROOT);
File::Find::find(\&process_file, $ARGV[0]);

##################################################
sub scan_include {            # Scan include files
##################################################
    my $file = $_;            # Save $_ 

    return unless -f $_;      # No directories

    open(FILE, "<$file") || 
        die "Cannot open $file (read)";

                            # relative path name
    (my $rel = $File::Find::name) =~
        s#^$INCLUDE_ROOT/*##g;

                            # read and store
    my $data = join('', <FILE>);
    chomp($data);
    $INCLUDE_MAP{"/$rel"} = $data;

    close(FILE);

    $_ = $file;             # reset $_
}

##################################################
sub process_file {
##################################################
    my $file = $_;

    return if -d $file;
    return unless $file =~ /\.html$/;

    open(FILE, "<$file") ||            # Read file
        die "Cannot open $file (read)";
    my $lines = join('', <FILE>);
    close(FILE);

    my $subs = ($lines =~        # Replace includes
        s@<!-- \s* include       # Intro tag
          \s+                    # Whitespace
          ([^\s]+)               # include file
          .*?-->                 # end of tag
          .*?<!--\s*/include\s*-->
         @include_replace($file, $1)@gsexi);
                                 # replace function

    if($subs) {
        print "$File::Find::name: $subs subs\n";
        
        if(!$opt{r}) {
            open(FILE, ">$file") || 
                die "Cannot open $file (write)";
            print FILE $lines; 
            close(FILE);
        }
    }

    $_ = $file;
}

##################################################
sub include_replace {
##################################################
    my ($file, $tag) = @_;

                     # Check if tag defined
     if(exists $INCLUDE_MAP{$tag}) {
                      # ... and return replacement
         return "<!-- include $tag -->" .
                "$INCLUDE_MAP{$tag}" .
                "<!-- /include -->";
     } else {
         die "Cannot resolve include '$tag' " .
             "in file ", cwd(), "/$file";
     }
 }
     
 ##################################################
 sub usage {
 ##################################################
     $0 =~ s#.*/##g;
     print "$0: @_.\n";
     print "usage: $0 " . 
           "[-r] [-i includedir] directory\n" .
           "-r: read only\n" .
           "-i: include file directory\n";
     exit 1;
 }

