#!/usr/bin/perl -w
##################################################
# chkdeja.pl - Check dejanews for new articles
# 
# Syntax: chkdeja.pl search_term
#
# 1999, mschilli@perlmeister.com
##################################################

my $PERSIST_FILE  = "/home/mschilli/chkdeja.dbm";
my $ENGINE = "http://www.dejanews.com/dnquery.xp";
my $MAX_PAGES     = 3;
my $HITS_PER_PAGE = 100;
my $GRACE_PERIOD  = 10;
my $VERSION       = "1.0";

use LWP::UserAgent;
use HTTP::Request::Common;
use HTML::TreeBuilder;
use GDBM_File;
use File::Basename;

    # Get command line argument
my $search_term = shift(@ARGV);
usage("Missing search term") unless $search_term;

    # Create persistant storage
tie(my %STORE, 'GDBM_File', $PERSIST_FILE, 
    &GDBM_WRCREAT, 0644) ||
   die "Cannot create GDBM file $PERSIST_FILE";

    # Build initial search request
my $url = URI::URL->new($ENGINE);

    # Dejanew's form parameters
$url->query_form(ST        => "PS", 
                 QRY       => $search_term,
                 defaultOp => "AND",
                 DBS       => 1,
                 format    => "terse",
                 showsort  => "date",
                 maxhits   => $HITS_PER_PAGE,
                 LNG       => "ALL",
                 subjects  => "",
                 groups    => "",
                 authors   => "",
                 fromdate  => "",
                 todate    => "",
                );

my $page = $url->as_string;

my $ua   = LWP::UserAgent->new();
$ua->agent("chkdeja/$VERSION");

my ($next_page, $new_hits);

do {                  # First page and followups
    $next_page = "";  # No 'next page' defined yet
    $new_hits  = 0;

        # Issue HTTP request
    my $response = $ua->request(GET $page);
    die("$page failed: ", $response->message) if 
        $response->is_error;

        # Parse content for new articles
    my $tree = HTML::TreeBuilder->new();
    $tree->parse($response->content);

        # Go through all <A> links found
    for(@{$tree->extract_links("a")}) {
        my ($url, $element) = @$_;

            # Grab 'next messages' link
        if($element->content()->[0] =~ 
           /next messages/i) {
            $next_page = $url;
        }

            # Check for message number
        if($url =~ /AN=(\d+)/) {
            unless(exists $STORE{"$search_term$1"}) {
                    # Print URL if not seen yet
                $STORE{"$search_term$1"} = 1;
                $new_hits++;
                print $element->content()->[0], 
                      "\n    $url\n";
            }
        }
    }

    $tree->delete();   # Clean up parse tree

    # Followup page? Continue!
} while(($page = $next_page) && 
        --$MAX_PAGES && 
        $new_hits == $HITS_PER_PAGE &&
        sleep($GRACE_PERIOD));
 
 untie(%STORE);
 
 ##################################################
 sub usage {
 ##################################################
     my $program = basename $0;
 
     print <<EOT;
 $program: @_
 usage: $program search_term
 EOT
     exit(0);
 }

