#!/usr/bin/perl -w
##################################################
# Monty-Hall simulation
# mschilli@perlmeister.com 1999
##################################################

my $NOF_GAMES = 10000;

foreach $switch (0..1) {
    for (1..$NOF_GAMES) {
        my $monty = Monty->new();
        $monty->pick();
        $monty->switch() if $switch;
        $outcome{$monty->call()}++;
    }
    printf "%-8s: Won=%4.2f%% Lost=%4.2f%%\n", 
           $switch ? "Switch" : "Stubborn",
           $outcome{1}*100/$NOF_GAMES,
           $outcome{0}*100/$NOF_GAMES;
    %outcome = ();
}

package Monty;

sub new {
    my $self = { doors => [qw(0 0 0)] };
    $self->{doors}->[int rand 3] = 1;
    bless $self, shift;
}

sub pick {
    my $self = shift;

    # Player's pick
    $self->{picked} = int rand 3;

    # Monty's pick
    foreach $door (0..2) {
        if($door != $self->{picked} &&
           !$self->{doors}->[$door]) { 
            $self->{opened} = $door;
            last;
        }
    }
}

sub switch {
    my $self = shift;
    foreach $door (0..2) {
        if($door != $self->{picked} && 
           $door != $self->{opened}) {
            $self->{picked} = $door;
            last;
        }
    }
}
    
sub call {
    my $self = shift;
    return($self->{doors}->[$self->{picked}]);
}

