#!/usr/bin/perl -w

my $NOF_WEIGHTS =  4;
my $NOF_DIFFS   = 40;
my $MAX_WEIGHT  = 40;

my $comb = Combine->new($MAX_WEIGHT, $NOF_WEIGHTS);
my $best = 0;

while(my @onecomb = $comb->next()) {
    @onecomb = map { $_ + 1 } @onecomb;
    my $diffs = calc_diffs(@onecomb);
    if($diffs > $best) {
        print join(',', @onecomb), " ($diffs)\n";
        $best = $diffs;
    }
    last if $diffs == $NOF_DIFFS;
}

sub calc_diffs {
    my @weights = @_;
    my %diffs   = ();

    my $scalecomb = Combine->new(3, $NOF_WEIGHTS);

    while(my @scale = $scalecomb->next()) {
        my $diff = 0;
        my $i    = 0;
        foreach my $scaleloc (@scale) {
            $diff += $weights[$i] if $scaleloc == 1;
            $diff -= $weights[$i] if $scaleloc == 2;
            $i++;
        }
        $diffs{abs($diff)}++ if $diff; 
    }
    return scalar keys %diffs;
}


package Combine;

sub new {
    my ($class, $n, $k) = @_;

    my $self = { count => 0,
                 total => $n ** $k };

    $self->{weights} = 
               [reverse map { $n**$_ } (0..$k-1)];

    bless $self, shift;
}

sub next {
    my $self   = shift;

    my $count  = $self->{count};
    my @result = ();

    return () if $count == $self->{total};

    foreach my $weight (@{$self->{weights}}) {
        my $val = int $count/$weight;
        push @result, $val;
        $count -= $val * $weight;
    }

    $self->{count}++;

    @result;
}

