#!/usr/bin/perl -w
##################################################
# Mirror a number of URLs on a file system
# 1999, mschilli@perlmeister.com
##################################################

use LWP::UserAgent;
use File::Basename;
use File::Path;
use File::Spec;

my $LOCAL_TREE = "/DATA/articles/LOCAL_TREE";
my $URLS_FILE  = "/DATA/articles/urls.txt";

### Read URLs from file, skip blanks and comments
open FILE, "<$URLS_FILE" or 
                     die "Cannot open $URLS_FILE";
@URLS = grep { ! /^\s*(#|$)/ } <FILE>;
chomp @URLS;
close FILE;

foreach $url (@URLS) {
    print mirror_url($url), "\n";
}

##################################################
### Transform a URL into a file name
##################################################
sub url2file {
    my $url = shift;
    $url =~ s#.*://##g;
    return File::Spec->catfile($LOCAL_TREE, $url);
}

##################################################
### Store content of a URL on disc if modified
##################################################
sub mirror_url {
    my $url  = shift;
    my $file = url2file($url);

    my $ua = LWP::UserAgent->new();

    print "Mirroring $url in $file\n";

    my $dir = dirname($file);
    mkpath($dir) unless -d $dir;

    my $response = $ua->mirror($url, url2file($url));
    if($response->is_error()) {
        print "Failed to get $url: ", 
              $response->message(), "\n";
    }
}

