##################################################
package XmlUserAgent;
##################################################

use LWP::UserAgent;
use URI::URL;
use XML::Parser;
use HTTP::Request::Common;

@ISA = qw(LWP::UserAgent);

##################################################
sub new {          # Constructor
##################################################
    my ($class) = @_;

    my $self  = { ua       => $class->SUPER::new(),
                  url      => undef, 
                  method   => undef,
                  headers  => [], 
                  params   => [],
                  username => undef,
                  password => undef,
                };

    bless $self, $class;
}

##################################################
sub get_basic_credentials {  
##################################################
    my $self = shift;

    return undef unless exists $self->{username};
    return($self->{username}, $self->{password}); 
}

##################################################
sub request {
##################################################
    my ($self, $req) = @_;

    return shift->SUPER::request(@_) if ref $req;

    my $start_handler = sub { 
                     handle_start($self, @_) };

    my $p = XML::Parser->new( 
        Handlers => { Start => $start_handler });
    $p->parse($req);

    if($self->{method} eq "POST") {
        $req = POST($self->{url}, $self->{params}, 
                    @{$self->{headers}});
    } else {
        # GET request -- assemble parameters
        $self->{url}->query_form(@{$self->{params}});
        $req = GET($self->{url}, 
                   @{$self->{headers}});
   }

   return $self->SUPER::request($req);
}

##################################################
sub handle_start {
##################################################
    my ($a, $p, $el, %atts) = @_;

    if($el eq "request") {
        $a->{method} = $atts{method} || "GET";
        $a->{url}    = URI::URL->new($atts{url});
        $a->{ua}->proxy($a->{method}, $atts{proxy}) if 
                            exists $atts{proxy};
    }

    if($el eq "credentials") {
        $a->{username} = $atts{username};
        $a->{password} = $atts{password};
    }

    if($el eq "parameter") {
        push(@{$a->{params}}, 
             $atts{name}, $atts{value});
    }

    if($el eq "header") {
        push(@{$a->{headers}}, 
             $atts{name}, $atts{value});
    }
}

