 package Sync;
 ##################################################
 # mschilli1@aol.com, 2000
 ##################################################
 
 use IO::File;
 use File::Find;
 use Digest::MD5 qw(md5_base64);
 
 my $STAT_FILE = ".syncstatus";
 
 ##################################################
 # $status = Sync->new(-basedir => "base/dir",
 #     -exclude => ["/excluded", '/dir/.*\.gif']);
 ##################################################
 sub new {
 ##################################################
     my ($class, %hash) = @_;
 
     my $self = { basedir => $hash{-basedir} || ".",
                  status  => {}
                };
 
     push( @{$hash{-exclude}}, $STAT_FILE);
 
     if(exists $hash{-exclude}) {
         $self->{exclude} = 
                  [map qr($_), @{$hash{-exclude}}];
     }
 
     bless $self, $class;
 }
 
 ##################################################
 sub read_status_file {
 ##################################################
     my $self = shift;
 
     my $file = "$self->{basedir}/$STAT_FILE";
 
     open STAT, "<$file" or return 0;
 
     while(<STAT>) {
         # MD5-Hash, timestamp, path
         if(/(\S+) (\d+) (.*)/) {
             $self->{status}->{$3} = [$1, $2];
         }
     }
 
     close STAT;
 
     return 1;
 }
 
 ##################################################
 sub update_status {
 ##################################################
     my($self) = @_;
 
     $self->{status_chk} = { %{$self->{status}} };
 
     find( sub { $self->finder }, $self->{basedir});
 
     # Everything that's left in status_chk is a
     # ref in the .syncstatus file without a 
     # corresponding real file -- axe them all!
     foreach my $path (keys %{$self->{status_chk}}) {
         delete $self->{status}->{$path};
     }
 }
 
 ##################################################
 sub finder {
 ##################################################
     my ($self)  = @_;
     my $file    = $_;
 
     return unless -f $file;
 
     my $path = "$File::Find::dir/$file";
     $path =~ s#^$self->{basedir}/##;
 
     foreach my $regex (@{$self->{exclude}}) {
         if($path =~ $regex) {
             $_ = $file; return 1;
         }
     }
 
     my $mod_time = (stat($file))[9];
 
     delete $self->{status_chk}->{$path} if 
         exists $self->{status}->{$path};
 
     if(!exists $self->{status}->{$path} ||
        $mod_time > $self->{status}->{$path}->[1]) {
             my $ctx = Digest::MD5->new();
             open FILE, "<$file" or 
                          die "Cannot open $file";
             $ctx->addfile(*FILE);
             close FILE;
             $self->{status}->{$path} = 
                       [$ctx->b64digest, $mod_time];
     }
 
     $_ = $file;
 }
 
 
 ##################################################
 sub write_status_file {
 ##################################################
     my $self = shift;
 
     my $path = "$self->{basedir}/$STAT_FILE";
 
     my $sfh = IO::File->new("> $path") or
         die "Cannot open $path";
 
     foreach my $path (keys %{$self->{status}}) {
         print $sfh "@{$self->{status}->{$path}}" .
                    " $path\n";
     }
 
     $sfh->close;
 }
 
 ##################################################
 # Compare
 ##################################################
 sub compare {
     my ($self, $remote) = @_;
 
     my @actions = ();
 
     foreach $path (keys %{$self->{status}}) {
 
         if(exists $remote->{status}->{$path}) {
 
             my ($local_md5, $local_time) = 
                       @{$self->{status}->{$path}};
             my ($remote_md5, $remote_time) = 
                     @{$remote->{status}->{$path}};
             
             next if $remote_md5 eq $local_md5;
 
             if($remote_time > $local_time) {
                 # Server got a newer version
                 push(@actions, 
                 [$path, "GPIDLR", "Remote newer"]);
             } else {
                 # Client got a newer version
                 push(@actions, 
                  [$path, "PGIDLR", "Local newer"]);
             }
         } else {
             # File's not on remote -- local only
             push(@actions, 
                  [$path, "PLI", "Local only"]);
         }
     }
 
     # Files on the remote server but not local:
 REMOTE:
     foreach $path (keys %{$remote->{status}}) {
 
         next if exists $self->{status}->{$path};
 
         foreach my $regex (@{$self->{exclude}}) {
             next REMOTE if $path =~ $regex;
         }
 
         push(@actions, 
              [$path, "GRI", "Remote only"]);
     }
 
     sort { $a->[0] cmp $b->[0] } @actions;
 }
 
 ##################################################
 sub stat_file {
 ##################################################
     return $STAT_FILE;
 }
 
 1;

