 #!/usr/bin/perl -w
 
 ##################################################
 my $LOCAL_DIR   = "/my/local/directory";
 my @EXCLUDE     = qw( .htaccess
                       .htpasswd
                       HTML/_
                     );
 my $REMOTE_HOST = "remote.host.com";
 my $REMOTE_DIR  = ".";
 my $USERNAME    = "ratbert";
 my $PASSWD      = "nixgibts!";
 my $PROMPT      = '/\$/';
 my $SERVERSYNC  = "syncserver.pl";
 my $GZIP        = "gzip";
 my $TELNET_TO   = 300;
 ##################################################
 
 use Sync;
 use Net::FTP;
 use Net::Telnet;
 use File::Basename;
 use File::Path;
 
 ##################################################
 # Read local tree
 ##################################################
 $local  = Sync->new(-basedir => $LOCAL_DIR, 
                     -exclude => \@EXCLUDE);
 $local->read_status_file() or warn "No status file";
 $local->update_status();
 $local->write_status_file();
 my $stat_file = $local->stat_file();
 
 ##################################################
 # First, run the sync.pl script on the server
 ##################################################
 print "Running $SERVERSYNC on $REMOTE_HOST ...\n";
 $telnet = new Net::Telnet (Timeout => $TELNET_TO,
                            Prompt  => $PROMPT);
 $telnet->open($REMOTE_HOST) or 
                    die "Cannot open $REMOTE_HOST";
 $telnet->login($USERNAME, $PASSWD) or 
                                die "Cannot login";
 $telnet->prompt($PROMPT);
 $telnet->cmd("cd $REMOTE_DIR; $SERVERSYNC"); 
 $telnet->cmd("$GZIP -9c $stat_file >$stat_file.gz");
 $telnet->close;
 
 ##################################################
 # Now, fetch the .syncstatus file from the server
 ##################################################
 print "Grabbing $stat_file from $REMOTE_HOST ...\n";
 $ftp = Net::FTP->new($REMOTE_HOST) or 
              die "Cannot connect to $REMOTE_HOST";
 $ftp->login($USERNAME, $PASSWD) or 
              die "Cannot login";
 $ftp->cwd($REMOTE_DIR) or 
              die "Cannot chddir to $REMOTE_DIR";
 $ftp->binary;
 $ftp->get("$stat_file.gz") or
              die "Cannot get $stat_file.gz";
 system("$GZIP -df $stat_file.gz");
 $ftp->quit;
 
 ##################################################
 # Read remote tree via status file
 ##################################################
 print "Reading in remote $stat_file ...\n";
 $remote = Sync->new(-exclude => \@EXCLUDE);
 $remote->read_status_file() or 
                       warn "No status file found";
 
 ##################################################
 # Compare and derive actions
 ##################################################
 @actions = $local->compare($remote);
 
 ##################################################
 # Put/Get files according to actions defined
 ##################################################
 $ftp = Net::FTP->new($REMOTE_HOST) or 
              die "Cannot connect to $REMOTE_HOST";
 $ftp->login($USERNAME, $PASSWD) or 
              die "Cannot login";
 $ftp->cwd($REMOTE_DIR) or 
              die "Cannot chdir to $REMOTE_DIR";
 $ftp->binary;
 
 print scalar @actions, " actions necessary.\n";
 
 foreach my $action (@actions) {
     my ($path, $choices, $reason) = @$action;
 
     my $local_dir  = dirname("$LOCAL_DIR/$path");
     my $local_file = basename($path);
 
     print "$path -- $reason\n";
     my $choice = ask_choice($choices);
     
     $choice eq 'g' and do {
         print "GET $path\n";
         mkpath($local_dir) unless -d $local_dir;
         $ftp->get($path, "$local_dir/$local_file") or
             die "GET failed" };
 
     $choice eq 'p' and do {
         print "PUT $path\n";
         $ftp->mkdir(dirname($path), 1);
         $ftp->put("$local_dir/$local_file", $path) or
             die "PUT failed" };
 
     $choice eq 'r' || $choice eq 'd' and do {
         print "DELETE remote $path\n";
         $ftp->delete($path) or die "Delete failed" };
 
     $choice eq 'l' || $choice eq 'd' and do {
         print "DELETE local $path\n";
         unlink "$local_dir/$local_file" or 
                                die "Unlink failed";
     };
 
     $choice eq 'i' and do { print "Ignoring\n" };
 
     print "\n\n";
 }
 
 $ftp->quit;
 
 ##################################################
 sub ask_choice {
 ##################################################
     my $choices = shift;
     my $default_action = substr($choices, 0, 1);
 
     local $| = 1;
 
     my %messages = (
         G => "[G]et Remote",
         P => "[P]ush Local",
         R => "[R]emote delete",
         L => "[L]ocal delete",
         D => "[D]elete local/remote",
         I => "[I]gnore" );
 
     {   while($choices =~ /(.)/g) {
             print $messages{$1}, "\n";
         }
         print "[$default_action]> ";
 
         chop($word = <STDIN>);
         $word = $default_action unless $word;
         redo unless exists $messages{uc($word)};
         return lc($word);
     }
 }

